//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.SearchWertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.InstrumentType;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.SearchResponse;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class FfmSearchWertpapier implements SearchWertpapier<FfmStockInfoBean> {

	/**
	 * Unsere Instanz
	 */
	private static FfmSearchWertpapier instance;

	/**
	 * privater Standardkonstruktor
	 */
	private FfmSearchWertpapier() {
		// absichtlich leer
	}

	/**
	 * Liefert die FfmSearchWertpapier Instanz.
	 *
	 * @return die FfmSearchWertpapier Instanz.
	 */
	public static FfmSearchWertpapier getIntance() {
		synchronized (FfmSearchWertpapier.class) {
			if (FfmSearchWertpapier.instance == null) {
				FfmSearchWertpapier.instance = new FfmSearchWertpapier();
			}
		}
		return FfmSearchWertpapier.instance;
	}

	/**
	 * @see de.frankmuenster.jameica.depot.finanzen.SearchWertpapier#search(java.lang.String)
	 */
	@Override
	public List<FfmStockInfoBean> search(final String name) {
		final List<FfmStockInfoBean> list = new ArrayList<>();
		URL url = null;
		try {
			final String encodedName = URLEncoder.encode(name, java.nio.charset.StandardCharsets.UTF_8.name());
			StringBuilder urlSB = new StringBuilder(130);
			urlSB.append(FfmWebAPI.SEARCH_URL).append(encodedName).append(FfmWebAPI.SEARCH_URL_PARAMS);
			final URL searchUrl = new URL(urlSB.toString());
			final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(searchUrl, true);
			final SearchResponse searchResponse = FfmWebAPI.MAPPER.readValue(responseString, SearchResponse.class);
			urlSB = new StringBuilder();
			final List<InstrumentType> inst = searchResponse.getInstruments();
			FfmStockInfoBean bean;
			for (final InstrumentType instrument : inst) {
				Logger.debug(instrument.getName());
				Logger.debug("ID : " + instrument.getInstrumentID()); //$NON-NLS-1$
				final String urlType = InstrumentTypeEnum.getUrl(instrument.getInstrumentType());
				if (urlType != null) {
					urlSB.setLength(0);
					urlSB.append(urlType).append('/').append(instrument.getUrlName());
					if (urlSB.length() > 0) {
						urlSB.insert(0, FfmWebAPI.MAIN_URL);
						url = new URL(urlSB.toString()); // NOPMD
						bean = new FfmStockInfoBean(); // NOPMD
						bean.setExchange("Brse FFM"); //$NON-NLS-1$
						// bean.setInstrumentId(instrument.getInstrumentID());
						bean.setName(instrument.getName());
						bean.setType(instrument.getInstrumentType());
						bean.setWkn(instrument.getWKN());
						bean.setIsin(instrument.getISIN());
						bean.setInstrumentId(Integer.toString(instrument.getInstrumentID()));
						bean.setUrl(url);
						list.add(bean);
					}
				}
			}
		} catch (IOException | FinanceException e) {
			Logger.error(Settings.getI18n().tr("FfmWertpapierHistory.createSearchURLFailed"), e); //$NON-NLS-1$
		}
		return list;
	}
}
