//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import java.net.URL;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.finanzen.StockInfoBean;

/**
 * Brse Frankfurt am Main Info Bean
 *
 * @author Frank Mnster
 */
public class FfmStockInfoBean extends StockInfoBean {

    /**
     * Instrument Typ
     */
    private String instrumentType;

    /**
     * Instument-ID
     */
    private String instrumentId;

    /**
     * ISIN nummer
     */
    private String isin;

    /**
     * URL zum Wertpapier
     */
    private URL url;

    /**
     * WKN
     */
    private String wkn;

    /**
     * Standartkonstruktor
     */
    public FfmStockInfoBean() {
        this(null, null, null, null);
    }

    /**
     * Returns a FfmStockInfoBean initialised with the given parameters.
     *
     * @param instrumentID
     *            the FFM Instrument ID
     * @param name
     *            the name of the stock
     * @param exchange
     *            the exchange
     * @param type
     *            the type of the stock
     */
    public FfmStockInfoBean(final String instrumentID, final String name, final String exchange, final String type) {
        super(name, exchange, type, null);
        this.instrumentId = instrumentID;
    }

    /**
     * @return the description build out of WKN and ISIN.
     */
    @Override
    public String getDescription() {

        final StringBuilder builder = new StringBuilder(30);
        builder.append("WKN: ").append(this.wkn).append("<br/>ISIN : ").append(this.isin); //$NON-NLS-1$//$NON-NLS-2$
        return builder.toString().replaceAll("null", Settings.getI18n().tr("FfmStockInfoBean.notAvailable")); //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * @return the instrumentType
     */
    public String getInstrumentType() {

        return this.instrumentType;
    }

    /**
     * @return the instrumentId
     */
    public String getInstrumentId() {

        return this.instrumentId;
    }

    /**
     * @return the instrumentId
     */
    public String getIsin() {

        return this.isin;
    }

    /**
     * @return the url
     */
    public URL getUrl() {

        return this.url;
    }

    /**
     * @return the wkn
     */
    public String getWkn() {

        return this.wkn;
    }

    /**
     * @param identifier
     *            the identifier to set
     */
    public void setInstrumentType(final String identifier) {

        this.instrumentType = identifier;
    }

    /**
     * @param instrumentId
     *            the instrumentId to set
     */
    public void setInstrumentId(final String instrumentId) {

        this.instrumentId = instrumentId;
    }

    /**
     * @param isin
     */
    public void setIsin(final String isin) {

        this.isin = isin;
    }

    /**
     * @param url
     *            the url to set
     */
    public void setUrl(final URL url) {

        this.url = url;
    }

    /**
     * @param wkn
     *            the wkn to set
     */
    public void setWkn(final String wkn) {

        this.wkn = wkn;
    }

    /**
     * @see Object#toString()
     */
    @Override
    public String toString() {

        final StringBuffer stringBuffer = new StringBuffer(super.toString());
        if (this.instrumentId != null) {
            stringBuffer.append('\t');
            stringBuffer.append(this.instrumentId);
        }
        if (this.instrumentType != null) {
            stringBuffer.append('\t');
            stringBuffer.append(this.instrumentType);
        }
        if (this.url != null) {
            stringBuffer.append('\t');
            stringBuffer.append(this.url);
        }
        return stringBuffer.toString();
    }

    /**
     * Liefert eine leeren DaxInfoBean
     *
     * @return the FfmStockBean with German DAX information
     */
    public static FfmStockInfoBean getDaxBean() {

        return new FfmStockInfoBean("2108198", "DAX 30", null, null); //$NON-NLS-1$ //$NON-NLS-2$
    }
}
