//
// Copyright 2014 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.jameica.depot.finanzen.boerseffm.json;

import java.util.ArrayList;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * JSON Antwort eine Such-Anfrage.
 * 
 * @author Frank Mnster
 *
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "Indizes", "Aktien", "Fonds", "Rohstoffe", "Anleihen" })
public class SearchResponse {

	/**
	 * Liste der gefundenen Indizes
	 */
	@JsonProperty("Indizes")
	private transient List<InstrumentType> indizes;
	/**
	 * Liste der gefundenen Aktien
	 */
	@JsonProperty("Aktien")
	private transient  List<InstrumentType> aktien;
	/**
	 * Liste der gefundenen Fonds 
	 */
	@JsonProperty("Fonds")
	private transient  List<InstrumentType> fonds  ;
	/**
	 * Liste der gefundenen Rohstoffe 
	 */
	@JsonProperty("Rohstoffe")
	private transient  List<InstrumentType> rohstoffe  ;
	/**
	 * Liste der gefundenen Anleihen 
	 */
	@JsonProperty("Anleihen")
	private transient  List<InstrumentType> anleihen  ;

	/**
	 * Standardkonstruktor
	 */
	public SearchResponse() {
		// absichtlich leer
	}

	/**
	 * Liefert alle gefundenen Instruments
	 * @return eine Liste mit allen gefundenen Instruments
	 */
	public List<InstrumentType> getInstruments() {
		
		final List<InstrumentType> allInstruments = new ArrayList<>();
		
		if (indizes != null) {
			allInstruments.addAll(indizes);
		}
		if (aktien != null) {
			allInstruments.addAll(aktien);
		}
		if (fonds != null) {
			allInstruments.addAll(fonds);
		}
		if (rohstoffe != null) {
			allInstruments.addAll(rohstoffe);
		}
		if (anleihen != null) {
			allInstruments.addAll(anleihen);
		}
		return allInstruments;
	}

	/**
	 * Liefert die gefundenen Aktien
	 * @return die gefundenen Aktien
	 */
	@JsonProperty("Aktien")
	public List<InstrumentType> getAktien() {
		return aktien;
	}

	/**
	 * Liefert die gefundenen Anleihen
	 * @return die gefundenen Anleihen
	 */
	@JsonProperty("Anleihen")
	public List<InstrumentType> getAnleihen() {
		return anleihen;
	}

	/**
	 * Liefert die gefundenen Fonds
	 * @return die gefundenen Fonds
	 */
	@JsonProperty("Fonds")
	public List<InstrumentType> getFonds() {
		return fonds;
	}

	/**
	 * Liefert die gefundenen Indizes
	 * @return die gefundenen Indizes
	 */
	@JsonProperty("Indizes")
	public List<InstrumentType> getIndizes() {
		return indizes;
	}

	/**
	 * Liefert die gefundenen Rohstoffe
	 * @return die gefundenen Rhostoffe
	 */
	@JsonProperty("Rohstoffe")
	public List<InstrumentType> getRohstoffe() {
		return rohstoffe;
	}

}
