//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.jameica.depot.finanzen.currencyexchangerate;

import java.math.BigDecimal;
import java.util.Date;

/**
 * Bean fr Wechselkurse
 *
 * @author Frank Mnster
 */
public class ExchangeRate {

    /**
     * Das Wechselkurs Datum
     */
    private Date date;

    /**
     * Der Wechselkurs
     */
    private BigDecimal rate;

    /**
     * Exaktes Datum
     */
    private boolean excatDate;

    /**
     * Standardkonstruktor
     *
     * @param rate
     * @param date
     */
    public ExchangeRate(final Date date, final BigDecimal rate) {
        this.date = date;
        this.rate = rate;
    }

    /**
     * Liefert das Datum des Wechselkurses
     *
     * @return das Datum des Wechselkurses
     */
    public Date getDate() {

        return this.date;
    }

    /**
     * Setzt das Datum des Wechselkurses
     *
     * @param date
     *            das zu setzende Datum des Wechselkurses
     */
    public void setDate(final Date date) {

        this.date = date;
    }

    /**
     * Liefert den Wechselkurs
     *
     * @return der Wechselkurs
     */
    public BigDecimal getRate() {

        return this.rate;
    }

    /**
     * Setzt den Wechselkurs
     *
     * @param rate
     *            der zu setzende Wechselkurs
     */
    public void setRate(final BigDecimal rate) {

        this.rate = rate;
    }

    /**
     * Liefert Angaben darber, ob das Wechselkursdatum dem angefragtem Datum
     * entspricht.
     * 
     * @return <i>true</i>, falls das Wechselkursdatum dem angefragtem Datum
     *         entspricht, <i>false</i> sonst.
     */
    public boolean isExcatDate() {

        return this.excatDate;
    }

    /**
     * Setzt die Angaben darber, ob das Wechselkursdatum dem angefragtem Datum
     * entspricht.
     * 
     * @param excatDate
     *            <i>true</i>, falls das Wechselkursdatum dem angefragtem Datum
     *            entspricht, <i>false</i> sonst.
     */
    public void setExcatDate(final boolean excatDate) {

        this.excatDate = excatDate;
    }

}
