package de.frankmuenster.jameica.depot.finanzen.fixer_io.json;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * JSON Reprsentation der Wechselkurseanwort bei FIXER.IO
 *
 * @author Frank Mnster
 *
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "base", "date", "rates" }) // NOPMD 
public class ForeignExchanges {

	/**
	 * Zugrundeliegende Whrung der Wechselkurse
	 */
	@JsonProperty("base")
	private String base;

	/**
	 * Datum der Wechselkurse
	 */
	@JsonProperty("date")
	private String date;

	/**
	 * Wechelkurse
	 */
	@JsonProperty("rates")
	private Rates rates;

	/**
	 * Standardkonstruktor
	 */
	public ForeignExchanges() {
		// Absichtlich leer
	}

	/**
	 * Liefert die zugrundeliegende Whrung
	 * 
	 * @return die zugrundeliegende Whrung
	 */
	@JsonProperty("base")
	public String getBase() {
		return this.base;
	}

	/**
	 * Setzt die zugrundeliegende Whrung
	 * 
	 * @param base
	 *            die zugrundeliegende Whrung
	 */
	@JsonProperty("base")
	public void setBase(final String base) {
		this.base = base;
	}

	/**
	 * Liefert das Datum der Wechselkurse
	 * 
	 * @return das Datum der Wechselkurse
	 */
	@JsonProperty("date")
	public String getDate() {
		return this.date;
	}

	/**
	 * Setzt das Datum der Wechselkurse
	 * 
	 * @param date
	 *            das Datum der Wechselkurse
	 */
	@JsonProperty("date")
	public void setDate(final String date) {
		this.date = date;
	}

	/**
	 * Liefert die Liste mit den Wechselkursen
	 * 
	 * @return Liste mit den Wechselkursen
	 */
	@JsonProperty("rates")
	public Rates getRates() {
		return this.rates;
	}

	/**
	 * Setzt die Liste mit den Wechselkursen
	 * 
	 * @param rates
	 *            Liste mit den Wechselkursen
	 */
	@JsonProperty("rates")
	public void setRates(final Rates rates) {
		this.rates = rates;
	}

}
