package de.frankmuenster.jameica.depot.finanzen.fixer_io.json;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.fasterxml.jackson.annotation.JsonAnyGetter;
import com.fasterxml.jackson.annotation.JsonAnySetter;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;

/**
 * JSON Reprsentation der Wechselkurse bei FIXER.IO
 *
 * @author Frank Mnster
 *
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class Rates {

	/**
	 * Map mit den vorhandenen Wechselkursen
	 */
	@JsonIgnore
	private final Map<String, Object> changeRates = new ConcurrentHashMap<String, Object>();

	/**
	 * Standardkonstruktor
	 */
	public Rates() {
		// Absichtlich leer
	}

	/**
	 * Liefert die Map mit den Wechselkursen
	 *
	 * @return Map mit den Wechselkursen
	 */
	@JsonAnyGetter
	public Map<String, Object> getChangeRates() {
		return this.changeRates;
	}

	/**
	 * Fgt einen Wechselkurs hinzu.
	 *
	 * @param name
	 *            Whrung
	 * @param value
	 *            Wechselkurs
	 */
	@JsonAnySetter
	public void setChangeRates(final String name, final Object value) {
		this.changeRates.put(name, value);
	}

}