/*
 * Copyright 2017 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.time.LocalDate;
import java.time.OffsetDateTime;

import de.frankmuenster.jameica.finanzen.StockInfoBean;

/**
 * Quandl Wertpapier Bean
 *
 * @author Frank Mnster
 *
 */
public class QuandlStockInfoBean extends StockInfoBean {

	/**
	 * Quandl Code
	 */
	private String code;

	/**
	 * ltester Wert
	 */
	private LocalDate oldestAvailable;

	/**
	 * Aktualisiert
	 */
	private OffsetDateTime refeshedAt;

	/**
	 * Jngster Wert
	 */
	private LocalDate newestAvailable;

	/**
	 * Standartkonstruktor
	 */
	public QuandlStockInfoBean() {
		this(null, null, null, null);
	}

	/**
	 * Konstruktor mit Angabe aller Felder.
	 *
	 * @param code
	 *            der Quandl Code
	 * @param name
	 *            der Wertpapiername
	 * @param exchange
	 *            Der B&ouml;rsenplatz
	 * @param type
	 *            Der Wertpapiertyp
	 */
	public QuandlStockInfoBean(final String code, final String name, final String exchange, final String type) {
		super(name, exchange, type, null);
		this.code = code;
	}

	/**
	 * Liefert den Quandl Code
	 *
	 * @return den Quandl Code
	 */
	public String getCode() {
		return this.code;
	}

	/**
	 * @return the oldestAvailable
	 */
	public LocalDate getOldestAvailable() {
		return this.oldestAvailable;
	}

	/**
	 * @return the refeshedAt
	 */
	public OffsetDateTime getRefeshedAt() {
		return this.refeshedAt;
	}

	/**
	 * @return the newestAvailable
	 */
	public LocalDate getNewestAvailable() {
		return this.newestAvailable;
	}

	/**
	 * Setzt den Quandl Code
	 *
	 * @param code
	 *            den Quandl Code
	 */
	public void setCode(final String code) {
		this.code = code;
	}

	/**
	 * @param oldestAvailable
	 *            the oldestAvailable to set
	 */
	public void setOldestAvailable(final LocalDate oldestAvailable) {
		this.oldestAvailable = oldestAvailable;
	}

	/**
	 * @param offsetDateTime
	 *            the refeshedAt to set
	 */
	public void setRefeshedAt(final OffsetDateTime offsetDateTime) {
		this.refeshedAt = offsetDateTime;
	}

	/**
	 * @param newestAvailable
	 *            the newestAvailable to set
	 */
	public void setNewestAvailable(final LocalDate newestAvailable) {
		this.newestAvailable = newestAvailable;
	}

	/**
	 * @see Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer stringBuffer = new StringBuffer(super.toString());
		if (this.code != null) {
			stringBuffer.append(this.code);
		}
		return stringBuffer.toString();
	}

}
