//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.util.List;

import com.jimmoores.quandl.DataSetRequest;
import com.jimmoores.quandl.TabularResult;
import com.jimmoores.quandl.classic.ClassicQuandlSession;
import com.jimmoores.quandl.util.QuandlRuntimeException;

import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.finanzen.WertpapierHistory;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class QuandlWertpapierHistory implements WertpapierHistory {

    /**
     * Das Auth Token
     */
    private final transient String authToken;

    /**
     * Der Quandl code
     */
    private final transient String code;

    /**
     * Konstruktor mit Angabe des Names
     *
     * @param code
     * @param authToken
     */
    public QuandlWertpapierHistory(final String code, final String authToken) {
	this.code = code;
	this.authToken = authToken;
    }

    /**
     * @see de.frankmuenster.jameica.finanzen.WertpapierHistory#getSourceName()
     */
    @Override
    public String getSourceName() {
	return "Quandle: " + this.code; //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.finanzen.WertpapierHistory#getValues()
     */
    @Override
    public List<HistoryValueBean> getValues() throws FinanceException {
	final ClassicQuandlSession session = MyQuandlSession.getSession(this.authToken);
	TabularResult result = null;
	try {
	    result = session.getDataSet(DataSetRequest.Builder.of(this.code).build());
	} catch (final QuandlRuntimeException e) {
	    Logger.error(e.getLocalizedMessage());
	}
	return HistoryReader.resultReader(this.code, result);
    }
}
