/* Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.controller;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.time.LocalDate;
import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.ExchangeRate;
import de.frankmuenster.jameica.depot.finanzen.fixer_io.FixerIOCurrencyConverter;
import de.frankmuenster.jameica.depot.gui.input.CurrencyInput;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.wertpapierorder.Order;
import de.frankmuenster.jameica.depot.wertpapierorder.OrderImpl;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.DateInput;
import de.willuhn.jameica.gui.input.DecimalInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Kontroller fuer eine Order in der Detailansicht.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass", "PMD.TooManyFields" })
public class OrderController extends AbstractControl implements SearchResultSelectListener<StockInfoBean> {

    /**
     * Die Wertpapier ID
     */
    private transient TextInput wpId;

    /**
     * Der Wertpapiername
     */
    private transient TextInput name;

    /**
     * Der Kauf-/Verkaufpreis
     */
    private transient DecimalInput price;

    /**
     * Die W&auml;hrung des Kauf-/Verkaufpreis.
     */
    private transient CurrencyInput stockCur;

    /**
     * Der Kauf-/Verkaufpreis in der Hauptw&auml;hrung.
     */
    private transient DecimalInput priceInMainCur;

    /**
     * Umsatzvolumen
     */
    private transient DecimalInput volume;

    /**
     * Die zus&auml;tzliche Kosten
     */
    private transient DecimalInput additionalCosts;

    /**
     * Die zus&auml;tzliche Kosten in der Hauptw&auml;hrung.
     */
    private transient DecimalInput addCostsInMainCur;

    /**
     * Die W&auml;hrung der zus&auml;tzliche Kosten
     */
    private transient CurrencyInput addCostsMainCur;

    /**
     * Kauf-/Verkaufdatum
     */
    private transient DateInput orderDate;

    /**
     * Kommentar
     */
    private transient TextAreaInput comment;

    /**
     * Das Order-Objekt
     */
    final private transient Order order;

    /**
     * Das zuge&ouml;rigige Wertpapier-Objekt
     */
    private transient Wertpapier wertpapier;

    /**
     * Die W&auml;hrung des Wertpapiers
     */
    private transient String wPapierWaehrung;

    /**
     * Ist es ein neues Wertpapier?
     */
    // private transient boolean neuesWertpapier;
    /**
     * Ist es eine neue Order
     */
    private transient boolean neueOrder;

    /**
     * Der Speicher-Button
     */
    private transient Button store;

    /**
     * Der Abbrechen-Button
     */
    private transient Button cancle;

    /**
     * Die Wertpapier historischen Daten als Feld, damit sie nur einmal geladen werden mssen.
     */
    private transient WertpapierHistoryBean wHistory;

    /**
     * @param view
     */
    public OrderController(final AbstractView view) {
        super(view);
        this.order = new OrderImpl();
        final Object currrentObject = this.getCurrentObject();
        if (currrentObject instanceof OrderHistory) {
            Logger.debug(Settings.getI18n().tr("OrderController.readStockOrderFromDbInfo")); //$NON-NLS-1$
            final OrderHistory orderHistory = (OrderHistory) this.getCurrentObject();
            try {
                this.wertpapier = orderHistory.getAktie();
                this.wPapierWaehrung = this.wertpapier.getCurrency();
                this.order.setAdditionalCosts(orderHistory.getAdditionalCosts());
                this.order.setAdditionalCostsInMainCurrency(orderHistory.getAdditionalCostsInMainCurrency());
                this.order.setAdditionalCostsCurrency(orderHistory.getAdditionalCostsCurrency());
                this.order.setName(orderHistory.getAktie().getName());
                this.order.setOrderDate(orderHistory.getOrderDate());
                this.order.setOrderPrice(orderHistory.getOrderPrice());
                this.order.setOrderPriceInMainCurrency(orderHistory.getOrderPriceInMainCurrency());
                this.order.setOrderVolume(orderHistory.getOrderAmount());
                this.order.setSymbol(orderHistory.getAktie().getArdId());
                this.order.setComment(orderHistory.getComment());
            } catch (final RemoteException e) {
                Logger.warn(Settings.getI18n().tr("OrderController.readStockOrderFromDbWarning", //$NON-NLS-1$
                        e.getMessage()));
            }
            // this.neuesWertpapier = false;
            this.neueOrder = false;
        } else if (currrentObject instanceof Wertpapier) {
            Logger.debug(Settings.getI18n().tr("OrderController.readStockFromDbCreateNewOrderInfo")); //$NON-NLS-1$
            this.wertpapier = (Wertpapier) currrentObject;
            try {
                this.wPapierWaehrung = this.wertpapier.getCurrency();
                this.order.setAdditionalCosts(0.0d);
                this.order.setAdditionalCostsCurrency(this.wPapierWaehrung);
                this.order.setSymbol(this.wertpapier.getArdId());
                this.order.setName(this.wertpapier.getName());
                this.order.setOrderVolume(0.0d);
                // Whrung und aktuellen Kurs ermitteln
                try {
                    ArdStockInfoBean wpInfoBean = new ArdStockInfoBean();
                    wpInfoBean.setId(this.wertpapier.getArdId());
                    wpInfoBean = BoerseArdApi.getInstance().getActualValue(wpInfoBean);
                    if (wpInfoBean != null) {
                        // neuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
                        this.order.setOrderDate(wpInfoBean.getDate());
                        this.order.setOrderPrice(wpInfoBean.getActValue().doubleValue());
                        this.handlePriceChanged();
                    }
                } catch (final FinanceException ye) {
                    // OK, dann halt nicht.
                    this.order.setOrderDate(new Date());
                    this.order.setOrderPrice(0.0d);
                }
            } catch (final RemoteException e) {
                Logger.error(Settings.getI18n().tr("OrderController.readStockFromDbCreateNewOrderError" //$NON-NLS-1$
                        , e.getMessage()));
            }
            // this.neuesWertpapier = false;
            this.neueOrder = true;
        }
    }

    /**
     * @return wpId
     */
    public TextInput getSymbol() {
        if (this.wpId == null) {
            this.wpId = new TextInput(this.order.getSymbol());
            // if (!this.isNeuesWertpapier()) {
            this.wpId.setEnabled(false);
            // }
        }
        return this.wpId;
    }

    /**
     * @return name
     */
    public TextInput getName() {
        if (this.name == null) {
            this.name = new TextInput(this.order.getName());
            // if (!this.isNeuesWertpapier()) {
            this.name.setEnabled(false);
            // }
        }
        return this.name;
    }

    /**
     * @return price
     */
    public DecimalInput getPrice() {
        if (this.price == null) {
            this.price = new DecimalInput(this.order.getOrderPrice(), FormatterUtils.DECIMAL_FORMAT_6.get());
            // if (StringUtils.isNotEmpty(wPapierWaehrung)) {
            // price.setComment(wPapierWaehrung);
            // } else {
            // price.setComment(""); // Platz lassen fr die //$NON-NLS-1$
            // // Whrung
            // }
            this.price.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    if (event.type == SWT.FocusIn) {
                        OrderController.this.price.hasChanged();
                    } else if (event.type == SWT.FocusOut && OrderController.this.price.hasChanged()) {
                        OrderController.this.handlePriceChanged();
                    }
                }
            });
        }
        return this.price;
    }

    /**
     * Whrung
     *
     * @return Whrungsauswahlfeld
     */
    public CurrencyInput getStockCurrency() {
        if (this.stockCur == null) {
            this.stockCur = new CurrencyInput(this.wPapierWaehrung);
            this.stockCur.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    Logger.warn(event.toString());
                    if (event.type == SWT.Selection) {
                        OrderController.this.wPapierWaehrung = (String) OrderController.this.stockCur.getValue();
                        OrderController.this.handleCostsChanged();
                    }
                }
            });
            // if (!this.neuesWertpapier) {
            this.stockCur.setEnabled(false);
            // }
        }
        return this.stockCur;
    }

    /**
     * @return priceInMainCur
     */
    public DecimalInput getPriceInMainCurrency() {
        if (this.priceInMainCur == null) {
            this.priceInMainCur = new DecimalInput(this.order.getOrderPriceInMainCurrency(),
                    FormatterUtils.DECIMAL_FORMAT_6.get());
            this.priceInMainCur.setComment(""); //$NON-NLS-1$
        }
        return this.priceInMainCur;
    }

    /**
     * @return volume
     */
    public DecimalInput getVolume() {
        if (this.volume == null) {
            this.volume = new DecimalInput(this.order.getOrderVolume(), FormatterUtils.DECIMAL_FORMAT_6.get());
        }
        return this.volume;
    }

    /**
     * @return additionalCosts
     */
    public DecimalInput getAdditionalCosts() {
        if (this.additionalCosts == null) {
            this.additionalCosts = new DecimalInput(this.order.getAdditionalCosts(),
                    FormatterUtils.DECIMAL_FORMAT_6.get());
            this.additionalCosts.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    if (event.type == SWT.FocusIn) {
                        OrderController.this.additionalCosts.hasChanged();
                    } else if (event.type == SWT.FocusOut && OrderController.this.additionalCosts.hasChanged()) {
                        OrderController.this.handleCostsChanged();
                    }
                }
            });
        }
        return this.additionalCosts;
    }

    /**
     * @return additionalCostsInMainCurrrency
     */
    public DecimalInput getAdditionalCostsInMainCurrency() {
        if (this.addCostsInMainCur == null) {
            this.addCostsInMainCur = new DecimalInput(this.order.getAdditionalCostsInMainCurrency(),
                    FormatterUtils.DECIMAL_FORMAT_6.get());
            this.addCostsInMainCur.setComment(StringUtils.EMPTY);
        }
        return this.addCostsInMainCur;
    }

    /**
     * @return additionalCosts
     */
    public CurrencyInput getAdditionalCostsCurrency() {
        if (this.addCostsMainCur == null) {
            this.addCostsMainCur = new CurrencyInput(this.order.getAdditionalCostsCurrency());
            this.addCostsMainCur.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    Logger.warn(event.toString());
                    if (event.type == SWT.Selection) {
                        OrderController.this.handleCostsChanged();
                    }
                }
            });
        }
        return this.addCostsMainCur;
    }

    /**
     * @return orderDate
     */
    public DateInput getOrderDate() {
        if (this.orderDate == null) {
            this.orderDate = new DateInput(this.order.getOrderDate(), FormatterUtils.GERMAN_DATE_FORMAT.get());
            this.orderDate.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    Logger.debug(event.toString());
                    if (event.type == SWT.Selection) {
                        OrderController.this.handleDateChanged();
                    }
                }
            });
        }
        return this.orderDate;
    }

    /**
     * @return comment
     */
    public TextAreaInput getComment() {
        if (this.comment == null) {
            this.comment = new TextAreaInput(this.order.getComment());
        }
        return this.comment;
    }

    /**
     * Liefert den Speichern-Button
     *
     * @return den Speichern-Button
     */
    public Button getStore() {
        if (this.store == null) {
            this.store = new Button(Settings.getI18n().tr("OrderController.saveBtn"), new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object context)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {
                    OrderController.this.handleStore();
                }
            }, null, true, "save-icon.png"); //$NON-NLS-1$
            this.store.setEnabled(true);
        }
        return this.store;
    }

    /**
     * Liefert den Abbrechen-Button
     *
     * @return den Abbrechen-Button
     */
    public Button getCancel() {
        if (this.cancle == null) {
            this.cancle = new Button(Settings.getI18n().tr(Settings.getI18n().tr("OrderController.cancelBtn")), //$NON-NLS-1$
                    new Action() {

                        /**
                         * @see Action#handleAction(Object context)
                         */
                        @Override
                        public void handleAction(final Object context) throws ApplicationException {
                            // if (OrderController.this.isNeuesWertpapier()) {
                            // GUI.startView(de.frankmuenster.jameica.depot.gui.views.DepotView.class,
                            // null);
                            // } else {
                            GUI.startView(de.frankmuenster.jameica.depot.gui.views.WertpapierDetailsView.class,
                                    OrderController.this.wertpapier);
                            // }
                        }
                    }, null, false, "cancle-icon.png"); //$NON-NLS-1$
        }
        return this.cancle;
    }

    // /**
    // * Ist es ein neues Wertpapier
    // *
    // * @return TRUE, falls es sich auch um ein neues Wertpapier handelt.
    // */
    // public boolean isNeuesWertpapier() {
    // return this.neuesWertpapier;
    // }
    /**
     * Ist es eine neue Order
     *
     * @return TRUE, falls es sich auch um eine neue Ordner handelt.
     */
    public boolean isNeueWertpapierOrder() {
        return this.neueOrder;
    }

    /**
     * Aktualisiert die Anzeige
     */
    public void updateView() {
        this.handleDateChanged();
        this.handlePriceChanged();
        this.handleCostsChanged();
    }

    private void handleStore() {
        // interne order Instance aktualisieren
        this.order.setAdditionalCosts((Double) this.getAdditionalCosts().getValue());
        this.order.setName((String) this.getName().getValue());
        this.order.setOrderDate((Date) this.getOrderDate().getValue());
        this.order.setOrderPrice((Double) this.getPrice().getValue());
        this.order.setOrderVolume((Double) this.getVolume().getValue());
        this.order.setSymbol((String) this.getSymbol().getValue());
        this.order.setOrderPriceInMainCurrency((Double) this.getPriceInMainCurrency().getValue());
        this.order.setAdditionalCostsCurrency((String) this.getAdditionalCostsCurrency().getValue());
        this.order.setAdditionalCostsInMainCurrency((Double) this.getAdditionalCostsInMainCurrency().getValue());
        try {
            final DBService service = Settings.getDBService();
            final Wertpapier wertpapier = this.getAktie(service, this.order.getSymbol());
            if (Logger.isLogging(Level.INFO)) {
                Logger.info(Settings.getI18n().tr("OrderController.storeOrderinfo") + this.order.toString()); // NOPMD //$NON-NLS-1$
            }
            OrderHistory orderHistory;
            // if (this.isNeuesWertpapier()) {
            // orderHistory = (OrderHistory)
            // service.createObject(OrderHistory.class, null);
            // } else
            // ist neue Order?
            if (this.isNeueWertpapierOrder()) {
                this.order.setComment((String) this.getComment().getValue());
                orderHistory = (OrderHistory) service.createObject(OrderHistory.class, null);
            } else {
                this.order.setComment((String) this.getComment().getValue());
                orderHistory = (OrderHistory) this.getCurrentObject();
            }
            orderHistory.setAktie(wertpapier);
            orderHistory.setOrderAmount(this.order.getOrderVolume());
            orderHistory.setOrderDate(this.order.getOrderDate());
            orderHistory.setOrderPrice(this.order.getOrderPrice());
            orderHistory.setOrderPriceInMainCurrency(this.order.getOrderPriceInMainCurrency());
            orderHistory.setAdditionalCosts(this.order.getAdditionalCosts());
            orderHistory.setAdditionalCostsCurrency(this.order.getAdditionalCostsCurrency());
            orderHistory.setAdditionalCostsInMainCurrency(this.order.getAdditionalCostsInMainCurrency());
            orderHistory.setComment(this.order.getComment());
            try {
                orderHistory.store();
                Application.getMessagingFactory()
                        .sendMessage(new StatusBarMessage(Settings.getI18n().tr("OrderController.storeOrderSuccess"), //$NON-NLS-1$
                                StatusBarMessage.TYPE_SUCCESS));
            } catch (final ApplicationException e) {
                Application.getMessagingFactory()
                        .sendMessage(new StatusBarMessage(e.getMessage(), StatusBarMessage.TYPE_ERROR));
            }
            // und zurck zur Wertpapier-Detailansicht
            GUI.startView(de.frankmuenster.jameica.depot.gui.views.WertpapierDetailsView.class, wertpapier);
        } catch (final RemoteException e) {
            Logger.error(Settings.getI18n().tr("OrderController.storeStockError"), e); //$NON-NLS-1$
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(
                            Settings.getI18n().tr("OrderController.storeStockErrorMsg", e.getMessage()), //$NON-NLS-1$
                            StatusBarMessage.TYPE_ERROR));
        } catch (final Exception e) { // NOPMD
            Logger.error(Settings.getI18n().tr("OrderController.storeStockError"), e); //$NON-NLS-1$
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(
                            Settings.getI18n().tr("OrderController.storeStockErrorMsg", e.getMessage()), //$NON-NLS-1$
                            StatusBarMessage.TYPE_ERROR));
        }
    }

    private Wertpapier getAktie(final DBService service, final String symbol) throws Exception { // NOPMD
        Wertpapier wertpapier = null;
        final DBIterator<?> aktien = service.createList(Wertpapier.class);
        while (aktien.hasNext()) {
            final Wertpapier wertpapierTest = (Wertpapier) aktien.next();
            Logger.warn(wertpapierTest.getArdId() + " - " + symbol); //$NON-NLS-1$
            if (wertpapierTest.getArdId().equals(symbol)) {
                wertpapier = wertpapierTest;
            }
        }
        if (wertpapier == null) {
            // Es muss ein neues WP angelegt werden.
            wertpapier = (Wertpapier) service.createObject(Wertpapier.class, null);
            wertpapier.setArdId(this.order.getSymbol());
            wertpapier.setName(this.order.getName());
            wertpapier.setCurrency(this.wPapierWaehrung);
            wertpapier.setComment((String) this.getComment().getValue());
            // Aktuellen Kurs ermitteln
            try {
                ArdStockInfoBean wpInfoBean = new ArdStockInfoBean(wertpapier.getArdId());
                wpInfoBean = BoerseArdApi.getInstance().getActualValue(wpInfoBean);
                if (wpInfoBean != null) {
                    // neuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
                    wertpapier.setLastTradeDate(wpInfoBean.getDate());
                    wertpapier.setLastTradePriceOnly(wpInfoBean.getActValue().doubleValue());
                }
            } catch (final FinanceException ye) {
                // OK, dann halt nicht.
                Logger.info(Settings.getI18n().tr("OrderController.noFinancialInfoFound")); //$NON-NLS-1$
            }
            wertpapier.store();
        }
        return wertpapier;
    }

    /**
     * @see SearchResultSelectListener#searchResultSelected(StockInfoBean)
     */
    @Override
    public void searchResultSelected(final StockInfoBean stockInfo) {
        Logger.warn(Settings.getI18n().tr("OrderController.searchResultSelectedInfo", stockInfo.toString())); //$NON-NLS-1$
        this.wpId.setValue(((ArdStockInfoBean) stockInfo).getId());
        this.name.setValue(stockInfo.getName());
        // Whrung und aktuellen Kurs ermitteln
        try {
            ArdStockInfoBean wpInfoBean = new ArdStockInfoBean(((ArdStockInfoBean) stockInfo).getId());
            wpInfoBean = BoerseArdApi.getInstance().getActualValue(wpInfoBean);
            if (wpInfoBean != null) {
                // neuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
                this.wPapierWaehrung = wpInfoBean.getCurrency();
                this.stockCur.setValue(this.wPapierWaehrung);
                this.price.setValue(wpInfoBean.getActValue().doubleValue());
                this.price.setComment(this.wPapierWaehrung);
                this.orderDate.setValue(wpInfoBean.getDate());
                this.handlePriceChanged();
            }
        } catch (final FinanceException ye) {
            // Melden, das Kurs nicht gefunden wurde
            final StringBuffer msg = new StringBuffer();
            msg.append(Settings.getI18n().tr("OrderController.stockInfoNotFoundMsg", (String) this.wpId.getValue())); //$NON-NLS-1$
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(msg.toString(), StatusBarMessage.TYPE_INFO));
            this.price.setValue(0.0d);
            this.price.setComment(""); //$NON-NLS-1$
            this.orderDate.setValue(null);
            this.wPapierWaehrung = Settings.getDepotWaehrung();
            this.handlePriceChanged();
        }
    }

    private void handleDateChanged() {
        final String symbol = (String) this.getSymbol().getValue();
        final LocalDate setDate = DateUtils.asLocalDate((Date) this.getOrderDate().getValue());
        if (StringUtils.isNotEmpty(symbol) && setDate != null) {
            Logger.debug(((Date) this.getOrderDate().getValue()).toString());
            // Versuch den Kurswert an dem Datum zu ermitteln
            try {
                if (this.wHistory == null || !this.wHistory.getwpId().equals(symbol)) {
                    this.wHistory = new WertpapierHistoryBean(this.wertpapier);
                }
                final HistoryValueBean value = this.wHistory.getValuesAtDate(setDate);
                if (value != null) {
                    this.price.setValue(value.getClose());
                    this.orderDate.setValue(DateUtils.asDate(value.getLocalDate()));
                    // Informiere Anwender falls Werte nicht taggenau
                    if (!setDate.isEqual(value.getLocalDate())) {
                        final String valueDateString = FormatterUtils.GERMAN_DATE_TIME_FORMATER.get()
                                .format(value.getLocalDate());
                        final String msg = Settings.getI18n().tr("OrderController.valueOnDateNotInHistory", //$NON-NLS-1$
                                valueDateString);
                        Application.getMessagingFactory()
                                .sendMessage(new StatusBarMessage(msg, StatusBarMessage.TYPE_INFO));
                    }
                    this.handlePriceChanged();
                }
            } catch (final FinanceException ye) {
                // Melden, das Kurs nicht gefunden wurde
                final StringBuffer msg = new StringBuffer();
                msg.append(Settings.getI18n().tr("OrderController.stockInfoNotFoundMsg", symbol)); //$NON-NLS-1$
                Application.getMessagingFactory()
                        .sendMessage(new StatusBarMessage(msg.toString(), StatusBarMessage.TYPE_INFO));
            }
        }
    }

    private void handlePriceChanged() {
        if (this.priceInMainCur != null && this.priceInMainCur.getControl() != null) {
            if (this.wPapierWaehrung.equals(Settings.getDepotWaehrung())) {
                this.priceInMainCur.setComment(StringUtils.EMPTY);
                this.priceInMainCur.getControl().setVisible(false);
            } else {
                if (StringUtils.isNotEmpty(this.wPapierWaehrung) && this.getOrderDate().getValue() != null) {
                    // priceInMainCur.getControl().setVisible(true);
                    final Date orderDate = (Date) this.getOrderDate().getValue();
                    try {
                        final ExchangeRate exRate = FixerIOCurrencyConverter.getInstance()
                                .exchangeRate(this.wPapierWaehrung, Settings.getDepotWaehrung(), orderDate);
                        final StringBuffer msg = new StringBuffer();
                        msg.append(Settings.getI18n().tr("Wechselkursdaten sind nicht tagaktuel")); //$NON-NLS-1$
                        Application.getMessagingFactory()
                                .sendMessage(new StatusBarMessage(msg.toString(), StatusBarMessage.TYPE_INFO));
                        final BigDecimal price = new BigDecimal(this.getPrice().getValue().toString());
                        final BigDecimal exchangedPrice = price.multiply(exRate.getRate());
                        this.getPriceInMainCurrency().setValue(exchangedPrice.doubleValue());
                        this.priceInMainCur.setComment(Settings.getI18n().tr("OrderController.in") //$NON-NLS-1$
                                + Settings.getDepotWaehrung());
                        this.priceInMainCur.getControl().setVisible(true);
                    } catch (final FinanceException e) {
                        if (Logger.isLogging(Level.INFO)) {
                            Logger.info(Settings.getI18n().tr("OrderController.exchangeRateNotFound", //$NON-NLS-1$
                                    e.getMessage()));
                        }
                    }
                } else {
                    this.priceInMainCur.setComment(StringUtils.EMPTY);
                    this.priceInMainCur.getControl().setVisible(false);
                }
            }
        }
    }

    private void handleCostsChanged() {
        if (((String) this.getAdditionalCostsCurrency().getValue()).equals(Settings.getDepotWaehrung())) {
            this.addCostsInMainCur.setComment(StringUtils.EMPTY);
            this.addCostsInMainCur.getControl().setVisible(false);
            return;
        }
        if (this.getAdditionalCostsCurrency().getValue() == null || this.getOrderDate().getValue() == null) {
            this.addCostsInMainCur.setComment(StringUtils.EMPTY);
            this.addCostsInMainCur.getControl().setVisible(false);
        } else {
            final Date orderDate = (Date) this.getOrderDate().getValue();
            try {
                final ExchangeRate exRate = FixerIOCurrencyConverter.getInstance().exchangeRate(
                        (String) this.getAdditionalCostsCurrency().getValue(), Settings.getDepotWaehrung(), orderDate);
                final StringBuffer msg = new StringBuffer();
                msg.append(Settings.getI18n().tr("Wechselkursdaten sind nicht tagaktuel")); //$NON-NLS-1$
                Application.getMessagingFactory()
                        .sendMessage(new StatusBarMessage(msg.toString(), StatusBarMessage.TYPE_INFO));
                final BigDecimal costs = new BigDecimal(this.getAdditionalCosts().getValue().toString());
                final BigDecimal exchangedCosts = costs.multiply(exRate.getRate());
                this.addCostsInMainCur.setValue(exchangedCosts.doubleValue());
                this.addCostsInMainCur.setComment(Settings.getI18n().tr("OrderController.in") //$NON-NLS-1$
                        + Settings.getDepotWaehrung());
                this.addCostsInMainCur.getControl().setVisible(true);
            } catch (final FinanceException e) {
                Logger.error(Settings.getI18n().tr("OrderController.exchangerateCalculationFailed", //$NON-NLS-1$
                        e.getMessage()));
            }
        }
    }
}
