//
// Copyright 2016 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlStockInfoBean;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.QuandlSearchPart;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;

/**
 * @author Frank Mnster
 */
public class QuandlDialogController implements SearchResultSelectListener<QuandlStockInfoBean> {

    /**
     * Der Bescheibungstext
     */
    private transient FormTextPart description;

    /**
     * Das QuandlCode Input
     */
    private transient TextInput quandlCodeInput;

    /**
     * Der Quandl Code
     */
    private transient String quandlCode;

    /**
     * Quandl-Suchfeld
     */
    private transient QuandlSearchPart quandlSearchPart;

    /**
     * Returns a new QuandlDialogController initialised with the given quandl
     * code
     *
     * @param quandlCode
     */
    public QuandlDialogController(final String quandlCode) {
        this.quandlCode = quandlCode;
    }

    /**
     * Liefert den Bereich, in der die Beschreibung angezeigt wird.
     *
     * @return FormTextPart fr die Beschreibung
     */
    public FormTextPart getDescription() {

        if (this.description == null) {
            this.description = new FormTextPart();
        }
        this.description = new FormTextPart("<form><p></p><p></p><p></p></form>"); //$NON-NLS-1$
        return this.description;
    }

    /**
     * Liefert eine TextInput fr den Quandl-Code
     *
     * @return quandlCodeInput TextInput fr den Quandl-Code
     * @throws RemoteException
     */
    public TextInput getQuandlCode() throws RemoteException {

        if (this.quandlCodeInput == null) {
            this.quandlCodeInput = new TextInput(this.quandlCode == null ? StringUtils.EMPTY : this.quandlCode);
        }
        return this.quandlCodeInput;
    }

    /**
     * Liefert eine QuandlSearchPart
     *
     * @return eine QuandlSearchPart
     */
    public QuandlSearchPart getQuandlSearchPart() {

        if (this.quandlSearchPart == null) {
            this.quandlSearchPart = new QuandlSearchPart(this, 15);
        }
        return this.quandlSearchPart;
    }

    /**
     * @see SearchResultSelectListener#searchResultSelected(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    public void searchResultSelected(final QuandlStockInfoBean bean) {

        this.quandlCode = bean.getCode();
        this.quandlCodeInput.setValue(this.quandlCode);
        final StringBuffer text = new StringBuffer("<form><p>"); //$NON-NLS-1$
        text.append(bean.getDescription()).append("</p></form>"); //$NON-NLS-1$

        try {
            this.description.setText(text.toString());
        } catch (final IllegalArgumentException e) {
            this.description.setText(bean.getDescription());
        }
        this.description.refresh();

    }

    /**
     * Liefert den aktuell ausgewhlten oder gesetzten QuandlCode.
     *
     * @return den aktuellen QuandlCode
     */
    public String getSelectedCode() {

        return this.quandlCode;
    }
}
