//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.gui.controller;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 * @param <I>
 * @param <S>
 */
public class SearchDialogController<I extends StockInfoBean, S extends AbstractStockSearchPart<I>>
        implements SearchResultSelectListener<I> {

    /**
     * Der Bescheibungstext
     */
    private transient FormTextPart description;

    /**
     * Das QuandlCode Input
     */
    private transient TextInput resultInput;

    /**
     * der Suchtext
     */
    private final transient String searchText;

    /**
     * Der InfoBean
     */
    private transient I bean;

    /**
     * Suchfeld Klasse
     */
    private final transient Class<S> searchPartClazz;

    /**
     * Suchfeld
     */
    private transient S searchPart;

    /**
     * Standardkonstruktor
     *
     * @param searchText
     * @param searchPartClazz
     */
    public SearchDialogController(final String searchText, final Class<S> searchPartClazz) {
        this.searchText = searchText;
        this.searchPartClazz = searchPartClazz;
    }

    /**
     * Liefert den Bereich, in der die Beschreibung angezeigt wird.
     *
     * @return FormTextPart fr die Beschreibung
     */
    public FormTextPart getDescription() {
        if (this.description == null) {
            this.description = new FormTextPart();
        }
        this.description = new FormTextPart("<form><p></p><p></p><p></p></form>"); //$NON-NLS-1$
        return this.description;
    }

    /**
     * Liefert eine TextInput fr die Ausgabe
     *
     * @return resultInput fr die Ausgabe
     * @throws RemoteException
     */
    public TextInput getResultInput() throws RemoteException {
        if (this.resultInput == null) {
            this.resultInput = new TextInput(this.searchText == null ? StringUtils.EMPTY : this.searchText);
        }
        return this.resultInput;
    }

    /**
     * Liefert das SearchPart
     *
     * @return das SearchPart
     * @throws ApplicationException
     */
    @SuppressWarnings("unchecked")
    public S getSearchPart() throws ApplicationException {
        if (this.searchPart == null) {
            try {
                final Constructor<?> cons = this.searchPartClazz.getConstructor(SearchResultSelectListener.class);
                final Object instance = cons.newInstance(this);
                if (instance instanceof AbstractStockSearchPart<?>) {
                    this.searchPart = (S) cons.newInstance(this);
                } else {
                    throw new InstantiationException(Settings.getI18n().tr("SearchDialogController.wrongInstanceType")); //$NON-NLS-1$
                }
            } catch (InstantiationException | IllegalAccessException | NoSuchMethodException | SecurityException
                    | IllegalArgumentException | InvocationTargetException e) {
                throw new ApplicationException(
                        Settings.getI18n().tr("SearchDialogController.errorCreatingSearchFieldInstance"), e); //$NON-NLS-1$
            }
        }
        return this.searchPart;
    }

    /**
     * Liefert das ausgewhlte Wertpapier.
     *
     * @return das ausgewhlte Wertpapier
     */
    public I getSelectedBean() {
        return this.bean;
    }

    /**
     * @see SearchResultSelectListener#searchResultSelected(StockInfoBean)
     */
    @Override
    public void searchResultSelected(final I stockInfo) {
        this.bean = stockInfo;
        if (this.bean != null) {
            this.resultInput.setValue(stockInfo.getName());
            final StringBuffer text = new StringBuffer("<form><p>"); //$NON-NLS-1$
            text.append(stockInfo.getDescription()).append("</p></form>"); //$NON-NLS-1$
            try {
                this.description.setText(text.toString());
            } catch (final IllegalArgumentException e) {
                this.description.setText(stockInfo.getDescription());
            }
            this.description.refresh();
        }
    }
}
