/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.controller;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.views.SettingsView;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.TextInput;

/**
 * Kontroller fr die Settings View
 *
 * @author munstfr
 * @version 1.0
 *
 */
public class SettingsController extends AbstractControl {

	/**
	 * Zeige die Hibiscus Konto Info's
	 */
	private transient CheckboxInput showHibiscusInfo;

	/**
	 * Quandl API
	 */
	private transient TextInput quandlApiKey;

	/**
	 * Erstellt eine neue Instanz des Kontrollers
	 *
	 * @param view
	 */
	public SettingsController(final SettingsView view) {
		super(view);
	}

	/**
	 * @return Ein CheckBoxInput zum setzten der Anzeige der
	 *         Hibiscuszusammenfassung.
	 */
	public CheckboxInput getShowHibiscusSummery() {
		if (this.showHibiscusInfo == null) {
			this.showHibiscusInfo = new CheckboxInput(Settings.isShowHibiscusSummery());
		}
		return this.showHibiscusInfo;
	}

	/**
	 * Liefert ein TextInput zur Eingabe des Quandl-API keys.
	 *
	 * @return ein TextInput zur Eingabe des Quandl-API keys
	 */
	public TextInput getQuandlApiKey() {

		if (this.quandlApiKey == null) {
			this.quandlApiKey = new TextInput(Settings.getQuandlApiKey());
			this.quandlApiKey.setComment(Settings.getI18n().tr("SettingsController.seeQuandlWebsite")); //$NON-NLS-1$
		}

		return this.quandlApiKey;
	}

	/**
	 * Speichert die aktuellen Einstellungen im Settings Objekt.
	 */
	public void speichern() {

		if (Settings.isHibiscusInstalled()) {
			Settings.setShowHibiscusSummery((Boolean) this.showHibiscusInfo.getValue());
		} else {
			Settings.setShowHibiscusSummery(false);
			
		}
		Settings.setQuandlApiKey((String) this.quandlApiKey.getValue());

		GUI.getStatusBar().setSuccessText(Settings.getI18n().tr("SettingsController.saveSuccess")); //$NON-NLS-1$

	}
}
