/*
 * Copyright 2017 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.dialog;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.controller.QuandlDialogController;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * Suchfenster fr Quandl-Codes
 *
 * @author Frank Mnster
 */
public class QuandlSearchDialog extends AbstractDialog<Object> {

    /**
     * Der ausgewhlte Code
     */
    private transient String code;

    /**
     * Unser Kontrolleur
     */
    private final transient QuandlDialogController controller;

    /**
     * Liefert einen modalen Dialog, der mittel .open() angezeigt werden kann
     *
     * @param position
     *            die Position, an der der Dialog angezeigt werden soll. Mgliche Werte sind POSITION_MOUSE und
     *            POSITION_CENTER.
     * @param code
     */
    public QuandlSearchDialog(final int position, final String code) {
        super(position, false);
        this.setTitle(Settings.getI18n().tr("QuandlSearchDialog.about")); //$NON-NLS-1$
        this.setPanelText(Settings.getI18n().tr("QuandlSearchDialog.panelText")); //$NON-NLS-1$
        this.controller = new QuandlDialogController(code);
        this.code = code;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected Object getData() {
        return this.code;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) throws RemoteException {
        final ColumnLayout columns = new ColumnLayout(parent, 2);
        final Container links = new SimpleContainer(columns.getComposite());
        // rechte Seite
        final Container rechts = new SimpleContainer(columns.getComposite(), true, 1);
        final GridData gridData = (GridData) rechts.getComposite().getLayoutData();
        gridData.minimumWidth = 200;
        gridData.widthHint = 200;
        // final GridLayout glImage = new GridLayout();
        // glImage.numColumns = 2;
        // container.getComposite().setLayout(glImage);
        // final GridData gdImage = new GridData();
        // gdImage.grabExcessHorizontalSpace = true;
        // gdImage.horizontalAlignment = SWT.RIGHT;
        // gdImage.horizontalSpan = 2;
        Part part = this.controller.getQuandlSearchPart();
        part.paint(links.getComposite());
        // Felder
        links.addLabelPair(Settings.getI18n().tr("QuandlSearchDialog.codeLbl"), this.controller.getQuandlCode()); //$NON-NLS-1$
        // Beschreibung
        rechts.addHeadline(Settings.getI18n().tr("QuandlSearchDialog.describtionHeader")); //$NON-NLS-1$
        part = this.controller.getDescription();
        part.paint(rechts.getComposite());
        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(Settings.getI18n().tr("QuandlSearchDialog.create"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {
                QuandlSearchDialog.this.code = QuandlSearchDialog.this.controller.getSelectedCode();
                QuandlSearchDialog.this.close();
            }
        }, null, true, "ok.png"); //$NON-NLS-1$
        buttons.addButton(Settings.getI18n().tr("QuandlSearchDialog.close"), new Action() {  //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {
                QuandlSearchDialog.this.close();
            }
        }, null, true, "window-close.png"); //$NON-NLS-1$
        final SimpleContainer footer = new SimpleContainer(parent);
        footer.addButtonArea(buttons);
        this.setSize(650, SWT.DEFAULT);
    }
}
