/*
 * Copyright 2017 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.formatter;

import java.text.DecimalFormat;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.formatter.Formatter;

/**
 * Formatierer fr Dezimalzahlen.
 *
 * @author Frank Mnster
 */
public class DecimalFormatter implements Formatter {

    /**
     * Das Decimalformat
     */
    private transient DecimalFormat formatter = FormatterUtils.DECIMAL_FORMAT_6.get();

    /**
     * Erzeugt einen neuen Formatierer mit dem angegeben Waehrungsstring.
     *
     * @param formatter
     *            kann optional angegeben werden, um den Betrag zu formatieren.
     *            Wird der Parameter weggelassen, werden die Werte auf 2 Stellen
     *            hinter dem Komma formatiert.
     */
    public DecimalFormatter(final DecimalFormat formatter) {

        if (formatter == null) {
            this.formatter.applyPattern("#0.00"); //$NON-NLS-1$
        } else {
            this.formatter = formatter;
        }
    }

    /**
     * Formatiert das bergebene Objekt. Es kann von folgenden Typen sein:
     * <ul>
     * <li>String</li>
     * <li>Number (oder davon abgeleitete Typen)</li>
     * </ul>
     *
     * @see de.willuhn.jameica.gui.formatter.Formatter#format(java.lang.Object)
     */
    @Override
    public String format(final Object objectToFormat) {

        String formatedText;
        if (objectToFormat == null) {
            formatedText = StringUtils.EMPTY;
        }
        if (objectToFormat instanceof Number) {
            formatedText = this.formatter.format(((Number) objectToFormat).doubleValue());
        } else {
            formatedText = objectToFormat.toString();
        }
        return formatedText;
    }

}

/*********************************************************************
 * $Log: DecimalFormatter.java,v $ Revision 1.3 2008/03/04 00:49:25 willuhn
 *
 * @N GUI fuer Backup fertig Revision 1.2 2004/04/27 00:04:44 willuhn
 * @D javadoc Revision 1.1 2004/04/12 19:15:59 willuhn
 * @C refactoring
 * @N forms Revision 1.3 2004/04/05 23:29:26 willuhn *** empty log message ***
 *    Revision 1.2 2004/03/11 08:56:55 willuhn
 * @C some refactoring Revision 1.1 2004/01/28 20:51:24 willuhn
 * @C gui.views.parts moved to gui.parts
 * @C gui.views.util moved to gui.util Revision 1.1 2003/12/29 20:07:19 willuhn
 * @N Formatter
 **********************************************************************/
