//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.gui.parts.wpsearch;

import java.rmi.RemoteException;
import java.util.AbstractMap.SimpleImmutableEntry;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.depot.gui.utils.SWTFontUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class BoerseArdSelectExchangePart implements Part {

    /**
     * Unsere Listeners fr SearchResultSelected
     */
    private final transient Set<ExchangeSelectListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Liste mit StockInfo's
     */
    protected transient Map<String, String> resultList = new ConcurrentHashMap<>();

    /**
     * Zeilen der Ergebnisliste
     */
    private transient int resultRows;

    /**
     * Die Liste mit den Suchergebnissen
     */
    protected transient org.eclipse.swt.widgets.List exchangeList;

    /**
     * Standardkonstruktor
     *
     * @param listner
     */
    public BoerseArdSelectExchangePart(final ExchangeSelectListener listner) {
        this.addListener(listner);
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            SearchResultSelectListener der hinzugefgt werden soll
     */
    public final void addListener(final ExchangeSelectListener listener) {
        this.listeners.add(listener);
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe dem gefundenen QuoteLookupInfoBean Element
     *
     * @param sel
     */
    protected void notifyListeners(final SimpleImmutableEntry<String, String> sel) {
        for (final ExchangeSelectListener listener : this.listeners) {
            listener.exchangeSelected(sel);
        }
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {
        if (parent == null) {
            Logger.error(I18N.tr("AbstractStockSearchPart.logParentIsNull")); //$NON-NLS-1$
            return;
        }
        final GridLayout gridLayout = new GridLayout(2, false);
        parent.setLayout(gridLayout);
        final Label searchResultLabel = new Label(parent, SWT.NONE);
        searchResultLabel.setText(I18N.tr("AbstractStockSearchPart.lblExchangeResults")); //$NON-NLS-1$
        GridData gridData = new GridData();
        gridData.verticalAlignment = SWT.BEGINNING;
        // gridData.horizontalSpan = 2;
        searchResultLabel.setLayoutData(gridData);
        this.exchangeList = new org.eclipse.swt.widgets.List(parent, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL);
        gridData = new GridData(GridData.FILL_BOTH);
        // gridData.horizontalSpan = 2;
        gridData.horizontalAlignment = SWT.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = SWT.FILL;
        gridData.grabExcessVerticalSpace = true;
        this.exchangeList.setLayoutData(gridData);
        Font font = GUI.getDisplay().getSystemFont();
        try {
            font = SWTFontUtils.getMonospacedFont();
        } catch (AssertionError | RuntimeException ex) { // NOPMD
            Logger.warn(I18N.tr("AbstractStockSearchPart.monospacedFontNotFound")); //$NON-NLS-1$
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(I18N.tr("AbstractStockSearchPart.monospacedFontNotFound"), //$NON-NLS-1$
                            StatusBarMessage.TYPE_INFO));
        }
        final FontData fontData = font.getFontData()[0];
        fontData.setHeight(8);
        font = new Font(parent.getDisplay(), fontData);
        this.exchangeList.setFont(font);
        // Gre festlegen
        for (int i = 0; i < this.resultRows; i++) {
            this.exchangeList.add(StringUtils.EMPTY);
        }
        this.exchangeList.addSelectionListener(new ExchangeSelectionListener());
    }

    /**
     * Listener zum Anzeigen, dass ein ein Suchergebnis angeklickt wurde.
     *
     * @author Frank Mnster
     */
    public interface ExchangeSelectListener {

        /**
         * Gibt das angeklickte StockInfo Element zurck
         *
         * @param exchange
         */
        void exchangeSelected(SimpleImmutableEntry<String, String> exchange);
    }

    /**
     * Auswahllistener der anzeigt, dass ein Suchergebniss angeklickt wurde.
     *
     * @author Frank Mnster
     */
    protected class ExchangeSelectionListener implements SelectionListener {

        /**
         * Standartkonstruktor
         */
        public ExchangeSelectionListener() {
            // absichtlich leer
        }

        /**
         * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org .eclipse.swt.events.SelectionEvent)
         */
        @Override
        public void widgetDefaultSelected(final SelectionEvent arg0) {
            // nichts zu tun hier
        }

        /**
         * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse .swt.events.SelectionEvent)
         */
        @Override
        public void widgetSelected(final SelectionEvent arg0) {
            final int index = BoerseArdSelectExchangePart.this.exchangeList.getSelectionIndex();
            final String exhange = BoerseArdSelectExchangePart.this.exchangeList.getItem(index);
            final SimpleImmutableEntry<String, String> sel = new SimpleImmutableEntry<>(exhange,
                    BoerseArdSelectExchangePart.this.resultList.get(exhange));
            BoerseArdSelectExchangePart.this.notifyListeners(sel);
        }
    }

    /**
     * Schreibt die Brsen in die Brsenliste. Alte Eintrge werden gelscht.
     *
     * @param exchanges
     *            die Brsen
     */
    public void fillList(final Map<String, String> exchanges) {
        this.exchangeList.removeAll();
        if (exchanges != null) {
            this.resultList = exchanges;
            for (final String exchange : exchanges.keySet()) {
                this.exchangeList.add(exchange);
            }
        }
    }
}
