/*
 * Copyright 2017 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.wpsearch;

import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmSearchWertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.jameica.gui.Part;

/**
 * Part das es erlaubt nach einem Wertpapier bei der Brese FFM zu suchen.
 *
 * @author Frank Mnster
 */
public class FfmSearchPart extends AbstractStockSearchPart<FfmStockInfoBean> implements Part {

    /**
     * Der Dialogtitel
     */
    private static final String DIALOG_TITLE = "Brese FFM Suche"; //$NON-NLS-1$

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     */
    public FfmSearchPart(final SearchResultSelectListener<FfmStockInfoBean> listener) {
        this(listener, 7);
    }

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     * @param resultRows
     */
    public FfmSearchPart(final SearchResultSelectListener<FfmStockInfoBean> listener, final int resultRows) {
        super(listener, resultRows);
    }

    /**
     * @see AbstractStockSearchPart#getTitle()
     */
    @Override
    public String getTitle() {
        return FfmSearchPart.DIALOG_TITLE;
    }

    /**
     * @see AbstractStockSearchPart#formatListEntry(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    protected String formatListEntry(final FfmStockInfoBean bean) {
        final StringBuffer result = new StringBuffer();
        StringBuffer help = new StringBuffer(bean.getName()).append(AbstractStockSearchPart.SPACE_20);
        result.append(help.substring(0, 20)).append(AbstractStockSearchPart.SPACE_3);
        help = new StringBuffer(bean.getType()).append(AbstractStockSearchPart.SPACE_10);
        result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
        final String wkn = bean.getWkn() == null ? StringUtils.EMPTY : bean.getWkn();
        help = new StringBuffer(wkn).append(AbstractStockSearchPart.SPACE_10);
        result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
        return result.toString();
    }

    /**
     * @see AbstractStockSearchPart#perfromSearch(java.lang.String)
     */
    @Override
    protected List<FfmStockInfoBean> perfromSearch(final String search) throws FinanceException {
        return FfmSearchWertpapier.getIntance().search(search);
    }
}
