/*
 * Copyright 2017 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.wpsearch;

import java.util.List;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlSearch;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlStockInfoBean;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.jameica.gui.Part;

/**
 * Part das es erlaubt nach einem Wertpapier in Quandl zu suchen.
 *
 * @author Frank Mnster
 */
public class QuandlSearchPart extends AbstractStockSearchPart<QuandlStockInfoBean> implements Part {

    /**
     * Der Dialogtitel
     */
    private static final String DIALOG_TITLE = "Quandl Suche"; //$NON-NLS-1$

    /**
     * Der Quandl API Key
     */
    private final transient String authToken;

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     */
    public QuandlSearchPart(final SearchResultSelectListener<QuandlStockInfoBean> listener) {
        this(listener, 7);
    }

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     * @param resultRows
     */
    public QuandlSearchPart(final SearchResultSelectListener<QuandlStockInfoBean> listener, final int resultRows) {
        super(listener, resultRows);
        this.authToken = Settings.getQuandlApiKey();
    }

    /**
     * @see AbstractStockSearchPart#getTitle()
     */
    @Override
    public String getTitle() {
        return QuandlSearchPart.DIALOG_TITLE;
    }

    /**
     * @see AbstractStockSearchPart#formatListEntry(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    protected String formatListEntry(final QuandlStockInfoBean bean) {
        final StringBuffer result = new StringBuffer();
        StringBuffer help = new StringBuffer(bean.getCode()).append(AbstractStockSearchPart.SPACE_10);
        result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
        help = new StringBuffer(bean.getExchange()).append(AbstractStockSearchPart.SPACE_6);
        result.append(help.substring(0, 6)).append(AbstractStockSearchPart.SPACE_3);
        help = new StringBuffer(bean.getName()).append(AbstractStockSearchPart.SPACE_30);
        result.append(help.substring(0, 30)).append(AbstractStockSearchPart.SPACE_3);
        // help = new StringBuffer(bean.getType()).append(SPACE_10);
        // result.append(help.substring(0, 10)).append(SPACE_3);
        return result.toString();
    }

    /**
     * @see AbstractStockSearchPart#perfromSearch(java.lang.String)
     */
    @Override
    protected List<QuandlStockInfoBean> perfromSearch(final String search) throws FinanceException {
        return QuandlSearch.getIntance(this.authToken).search(search);
    }
}
