//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europäischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie dürfen dieses Werk ausschließlich gemäß
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
// ausdrücklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschränkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.gui.views;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.layout.GridData;

import de.frankmuenster.jameica.depot.gui.controller.EditWertpapierController;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Münster
 */
public class EditWertpapierView extends AbstractView {

    /**
     * Der Kontrolleur
     */
    private transient EditWertpapierController control;

    /**
     * Standardkonstruktor
     */
    public EditWertpapierView() {
        super();
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws Exception {
        GUI.getView().setTitle(I18N.tr("EditWertpapierView.titel")); //$NON-NLS-1$
        this.control = new EditWertpapierController(this);
        // layout with 2 columns
        final ColumnLayout columns = new ColumnLayout(this.getParent(), 2);
        final Container links = new SimpleContainer(columns.getComposite(), false, 2);
        // Suchfenster
        // rechte Seite
        final Container right = new SimpleContainer(columns.getComposite(), true);
        GridData gridData = (GridData) right.getComposite().getLayoutData();
        gridData.minimumWidth = 250;
        right.addHeadline(I18N.tr("_description")); //$NON-NLS-1$
        right.addInput(this.control.getComment());
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        // Linke Seite
        // Anzeigen nur, wenn neues Wertpapier
        links.addHeadline(I18N.tr("EditWertpapierView.search")); //$NON-NLS-1$
        links.addPart(this.control.getBoerseArdSearchPart());
        links.addPart(this.control.getBoerseArdSelectExchangePart());
        links.addHeadline(I18N.tr("EditWertpapierView.ardInfo")); //$NON-NLS-1$
        links.addLabelPair(I18N.tr("EditWertpapierView.identifier"), this.control.getSymbol()); //$NON-NLS-1$
        links.addLabelPair(I18N.tr("EditWertpapierView.wpName"), this.control.getName()); //$NON-NLS-1$
        links.addText(StringUtils.EMPTY, true);
        // Whrung
        links.addHeadline(I18N.tr("_currency")); //$NON-NLS-1$
        links.addLabelPair(I18N.tr("_currency"), this.control.getStockCurrency()); //$NON-NLS-1$
        links.addText(StringUtils.EMPTY, true);
        // Historische Daten
        links.addHeadline(I18N.tr("EditWertpapierView.historicalSearchParameter")); //$NON-NLS-1$
        final SimpleContainer container = new SimpleContainer(links.getComposite(), false, 3);
        gridData = new GridData();
        gridData.horizontalSpan = 2;
        gridData.horizontalAlignment = GridData.FILL;
        container.getComposite().setLayoutData(gridData);
        // FFM Instrument
        container.addLabelPair(I18N.tr("EditWertpapierView.ffmInstrument"), //$NON-NLS-1$
                this.control.getFfmInstrument());
        this.control.getFfmSearchBtn().paint(container.getComposite());
        // Quandl Code
        container.addLabelPair(I18N.tr("EditWertpapierView.quandlCode"), this.control.getQuandlCode()); //$NON-NLS-1$
        final Button quandlSearch = this.control.getQuandlSearchBtn();
        if (quandlSearch != null) {
            quandlSearch.paint(container.getComposite());
        }
        // Speicher und Abbrechen Buttons einfügen
        final ButtonArea buttons = new ButtonArea();
        final Button store = this.control.getStore();
        final Button cancel = this.control.getCancel();
        buttons.addButton(store);
        buttons.addButton(cancel);
        buttons.paint(this.getParent());
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#unbind()
     */
    @Override
    public void unbind() throws ApplicationException {
        super.unbind();
        this.control = null; // NOPMD
    }
}
