/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse.BollingBaender; // NOPMD

import java.time.LocalDate;
import java.util.LinkedList;
import java.util.Queue;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * Bolling B&auml;nder
 *
 * @author Frank Mnster
 */
public class BollingBaender extends AbstractKursAnalyse<Integer, KursDaten[]> {

    /**
     * Methodenname
     */
    private final static String NAME = "BOLLING"; //$NON-NLS-1$

    /**
     * Bollig-B&auml;nder mit Average 10
     */
    private final static int BOLLIG_10 = 10;

    /**
     * Bollig-B&auml;nder mit Average 50
     */
    private final static int BOLLIG_50 = 50;

    /**
     * Tempor&auml;rge Queue zum Zwischenspeichern von Werten
     */
    private final transient Queue<Double> temp = new LinkedList<>();

    /**
     * Summe
     */
    private transient double sum;

    /**
     * Daten der Bollig Mittellinie (Trigger)
     */
    private transient KursDaten bollingMitte;

    /**
     * Daten der oberen Bolliglinie
     */
    private transient KursDaten bollingOben;

    /**
     * Daten der unteren Bolliglinie
     */
    private transient KursDaten bollingUnten;

    /**
     * Standardkonstruktor, ruft nur super auf.
     */
    public BollingBaender() {
        super();
    }

    /**
     * Konstruktor mit Angabe der Kursdaten und der zu benutzen Parameter.
     *
     * @param wpHistoryBean
     *            dir Kursdaten
     * @param parameter
     *            die Paramater
     */
    public BollingBaender(final WertpapierHistoryBean wpHistoryBean, final KursAnalyseParameter<Integer> parameter) {
        super(wpHistoryBean, parameter);
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {
        return BollingBaender.NAME;
    }

    /**
     * Liefert die AddAction
     *
     * @return die BollingBaenderHandleAddAction
     */
    public static Action getHandleAction() {
        return new BollingBaenderHandleAddAction();
    }

    /**
     * @see AbstractKursAnalyse#getResult(LocalDate, LocalDate)
     */
    @Override
    public KursDaten[] getResult(final LocalDate fromDate, final LocalDate toDate) {
        if (this.bollingMitte == null) {
            this.calculate();
        }
        // Ergbniss zusammenkrzen auf Zeitangebe
        final KursDaten bbMitte = new KursDaten();
        final KursDaten bbOben = new KursDaten();
        final KursDaten bbUnten = new KursDaten();
        for (int i = 0; i < this.bollingMitte.size(); i++) {
            final DateValuePair bbMitteValue = this.bollingMitte.get(i);
            if (DateUtils.isInRange(bbMitteValue.getDate(), fromDate, toDate)) {
                final DateValuePair bbObenValue = this.bollingOben.get(i);
                final DateValuePair bbUntenValue = this.bollingUnten.get(i);
                bbMitte.add(bbMitteValue);
                bbOben.add(bbObenValue);
                bbUnten.add(bbUntenValue);
            }
        }
        // nun das Ergebnis zusammenbasteln
        final KursDaten[] result = new KursDaten[3];
        result[0] = bbMitte;
        result[1] = bbOben;
        result[2] = bbUnten;
        return result;
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel ()
     */
    @Override
    public String getLegendTitel() {
        return "Bolling [" + this.parameter.getValue() + "]"; //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * Hier werden die Bollig-B&auml;nderdaten berechnet.
     */
    private void calculate() {
        this.bollingMitte = new KursDaten();
        this.bollingOben = new KursDaten();
        this.bollingUnten = new KursDaten();
        double faktor = 2.0;
        if (this.parameter.getValue() == BollingBaender.BOLLIG_10) {
            faktor = 1.9;
        } else if (this.parameter.getValue() == BollingBaender.BOLLIG_50) {
            faktor = 2.1;
        }
        for (int i = 0; i < this.closeDaten.size(); i++) {
            final DateValuePair orgValue = this.closeDaten.get(i);
            this.add(orgValue.getValue());
            final double bollingMitteValue = this.getAverage();
            final double stdAbweichung = this.getStandardAbweichung(bollingMitteValue);
            final double bollingObenValue = bollingMitteValue + stdAbweichung * faktor;
            final double bollingUntenValue = bollingMitteValue - stdAbweichung * faktor;
            if (!Double.isNaN(bollingMitteValue)) {
                this.bollingMitte.add(new DateValuePair(orgValue.getDate(), // NOPMD
                        bollingMitteValue));
                this.bollingOben.add(new DateValuePair(orgValue.getDate(), // NOPMD
                        bollingObenValue));
                this.bollingUnten.add(new DateValuePair(orgValue.getDate(), // NOPMD
                        bollingUntenValue));
            }
        }
    }

    /**
     * Berechnet die Standardabweichung des Mittelbandes f&uuml;r den angegeben Zeitraum.
     *
     * @param bollingMitte
     * @return die Standardabweichung
     */
    private double getStandardAbweichung(final double bollingMitte) {
        Double stdAbweichung = Double.NaN;
        if (this.temp.size() >= this.parameter.getValue()) {
            double sumAbweichungen = 0.0;
            for (final double close : this.temp) {
                sumAbweichungen += (close - bollingMitte) * (close - bollingMitte);
            }
            stdAbweichung = Math.sqrt(sumAbweichungen / this.temp.size());
        }
        return stdAbweichung;
    }

    /**
     * F&uuml;gt einen Wert in die Queue. Wurde die Zeitspane aus Parameter erreicht, wird der &auml;lteste Wert aus der
     * Queue gel&ouml;scht. Die Summe sum wird ebenfalls immer aus letzetn n Werten (aus Parameter) aktualisiert.
     *
     * @param wert
     */
    private void add(final double wert) {
        this.sum += wert;
        this.temp.add(wert);
        if (this.temp.size() > this.parameter.getValue()) {
            this.sum -= this.temp.remove();
        }
    }

    /**
     * Liefert den Durchschnitt der letzten n-Werte (wenn auch mindestens n-Werte vorhanden sind.
     *
     * @return der Durchschnitt der letzten n-Werte
     */
    private double getAverage() {
        Double durchschnitt = Double.NaN;
        if (this.temp.size() >= this.parameter.getValue()) {
            durchschnitt = this.sum / this.temp.size();
        }
        return durchschnitt;
    }
}
