/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse.EMA; // NOPMD 

import java.time.LocalDate;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * ExponentialMovingAverage
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class ExponentialMovingAverage extends AbstractKursAnalyse<Integer, KursDaten[]> {

    /**
     * Methodenname
     */
    private final static String NAME = "EMA"; //$NON-NLS-1$

    /**
     * ExponentialMovingAverage Daten
     */
    private transient KursDaten ema;

    /**
     * Konstruktor der nurt super() aufruft.
     */
    public ExponentialMovingAverage() {
        super();
    }

    /**
     * Konstruktor mit Angabe der Kursdaten und des Parameters.
     *
     * @param wpHistoryBean
     * @param parameter
     */
    public ExponentialMovingAverage(final WertpapierHistoryBean wpHistoryBean,
            final KursAnalyseParameter<Integer> parameter) {
        super(wpHistoryBean, parameter);
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {
        return ExponentialMovingAverage.NAME;
    }

    /**
     * Liefert die AddAction
     *
     * @return die EmaHandleAddAction
     */
    public static Action getHandleAction() {
        return new EmaHandleAddAction();
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel ()
     */
    @Override
    public String getLegendTitel() {
        return "EMA" + this.parameter.getValue(); //$NON-NLS-1$
    }

    /**
     * @see AbstractKursAnalyse#getResult(LocalDate, LocalDate)
     */
    @Override
    public KursDaten[] getResult(final LocalDate fromDate, final LocalDate toDate) {
        if (this.ema == null) {
            this.callculate();
        }
        final KursDaten result[] = new KursDaten[1];
        final KursDaten emaSeries = new KursDaten();
        for (int i = 0; i < this.ema.size(); i++) {
            final DateValuePair orgValue = this.ema.get(i);
            if (DateUtils.isInRange(orgValue.getDate(), fromDate, toDate)) {
                emaSeries.add(orgValue);
            }
        }
        result[0] = emaSeries;
        return result;
    }

    /**
     * Hier wird gerechnet
     */
    private void callculate() {
        // Das Intervall
        final Integer interval = this.parameter.getValue();
        // EMA Daten
        this.ema = new KursDaten();
        double expWight;
        expWight = 2.0 / (interval + 1.0);
        this.ema.add(new DateValuePair(this.closeDaten.get(0).getDate(), this.closeDaten.get(0).getValue()));
        for (int i = 1; i < this.closeDaten.size(); i++) {
            final double emaValue = this.ema.get(i - 1).getValue()
                    + expWight * (this.closeDaten.get(i).getValue() - this.ema.get(i - 1).getValue());
            this.ema.add(new DateValuePair(this.closeDaten.get(i).getDate(), emaValue)); // NOPMD
        }
    }
}
