/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse.MOM; // NOPMD 

import java.awt.Font;
import java.time.LocalDate;

import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SegmentedTimeline;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.willuhn.jameica.gui.Action;
import de.willuhn.logging.Logger;

/**
 * Momentum Methode
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class Momentum extends AbstractKursAnalyse<MomentumParameter, XYPlot> {

    /**
     * Methodenname
     */
    private final transient static String NAME = "Momentum"; //$NON-NLS-1$

    /**
     * Werte der Serie
     */
    private transient TimeSeries serieMOM;

    /**
     * Der Rendere
     */
    private transient XYItemRenderer rendererMOM;

    /**
     * Standardkonstruktor
     */
    public Momentum() {
        super();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {
        return Momentum.NAME;
    }

    /**
     * Liefert die AddAction
     *
     * @return die MomentumHandleAddAction
     */
    public static Action getHandleAction() {
        return new MomentumHandleAddAction();
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {
        final MomentumParameter para = this.parameter.getValue();
        final int period = para.getPeriod();
        return "Momentum (" + period + ")"; //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * @see AbstractKursAnalyse#getResult(LocalDate, LocalDate)
     */
    @Override
    public XYPlot getResult(final LocalDate fromDate, final LocalDate toDate) {
        if (this.anaG == null) {
            final DateAxis dateAxisKurs = new DateAxis(Settings.getI18n().tr("Momentum.axisDate")); //$NON-NLS-1$
            final NumberAxis rangeAxisKurs = new NumberAxis(Settings.getI18n().tr("Momentum.axisNumber")); //$NON-NLS-1$
            rangeAxisKurs.setAutoRangeIncludesZero(false);
            this.anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
            ((DateAxis) this.anaG.getDomainAxis()).setTimeline(SegmentedTimeline.newMondayThroughFridayTimeline());
            this.anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
            // XYAreaRenderer rendererMACD = new XYAreaRenderer();
            this.rendererMOM = new StandardXYItemRenderer();
            this.rendererMOM.setSeriesPaint(0, java.awt.Color.BLACK);
            final LegendTitle legendTitle = new LegendTitle(this.anaG);
            legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
            legendTitle.setBackgroundPaint(new java.awt.Color(200, 200, 255, 100));
            legendTitle.setFrame(new BlockBorder(java.awt.Color.white));
            legendTitle.setPosition(RectangleEdge.BOTTOM);
            final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
            legendTitle.setItemLabelPadding(padding);
            final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(1.0, 0.00, legendTitle,
                    RectangleAnchor.BOTTOM_RIGHT);
            titleAnnotation.setMaxWidth(0.48);
            this.anaG.addAnnotation(titleAnnotation);
        }
        if (this.serieMOM == null) {
            this.callculate();
        }
        final Day fromDay = new Day(DateUtils.asDate(fromDate));
        final Day toDay = new Day(DateUtils.asDate(toDate));
        try {
            final TimeSeriesCollection tmpMom = new TimeSeriesCollection(this.serieMOM.createCopy(fromDay, toDay));
            this.anaG.setDataset(0, tmpMom);
            this.anaG.setRenderer(0, this.rendererMOM);
        } catch (final CloneNotSupportedException e) {
            Logger.error(Settings.getI18n().tr("Momentum.createTsDataError"), //$NON-NLS-1$
                    e);
        }
        return this.anaG;
    }

    /**
     * Nun mal rechnen
     */
    private void callculate() {
        final int period = this.parameter.getValue().getPeriod();
        this.serieMOM = new TimeSeries(this.getLegendTitel());
        Day day = new Day(DateUtils.asDate(this.closeDaten.get(0).getDate()));
        this.serieMOM.add(day, 0.0);
        for (int i = 1; i < this.closeDaten.size(); i++) {
            double newValue = Double.NaN;
            final double value = this.closeDaten.get(i).getValue();
            day = new Day(DateUtils.asDate(this.closeDaten.get(i).getDate())); // NOPMD
            if (i >= period) {
                newValue = value - this.closeDaten.get(i - period).getValue();
            }
            this.serieMOM.add(day, newValue);
        }
    }
}
