/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse.SMA; //NOPMD

import java.time.LocalDate;
import java.util.LinkedList;
import java.util.Queue;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * SimpleMovingAverage Methode
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class SimpleMovingAverage extends AbstractKursAnalyse<Integer, KursDaten[]> {

    /**
     * Methodenname
     */
    private final static String NAME = "SMA"; //$NON-NLS-1$

    /**
     * Queue Objekt zur einfachen Aufbewahrung der letzten n Werte.
     */
    private final transient Queue<Double> temp = new LinkedList<>();

    /**
     * Gloables Summen Objekt zur Berechnung der Durchschnittswerte.
     */
    private transient double sum;

    /**
     * Standardkonstruktor
     */
    public SimpleMovingAverage() {
        super();
    }

    /**
     * @param wpHistoryBean
     * @param parameter
     */
    public SimpleMovingAverage(final WertpapierHistoryBean wpHistoryBean,
            final KursAnalyseParameter<Integer> parameter) {
        super(wpHistoryBean, parameter);
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {
        return SimpleMovingAverage.NAME;
    }

    /**
     * Liefert die Add Aktion
     *
     * @return die SimpleMovingAverageHandleAddAction
     */
    public static Action getHandleAction() {
        return new SimpleMovingAverageHandleAddAction();
    }

    /**
     * @see AbstractKursAnalyse#getResult(LocalDate, LocalDate)
     */
    @Override
    public KursDaten[] getResult(final LocalDate fromDate, final LocalDate toDate) {
        final KursDaten result[] = new KursDaten[1];
        final KursDaten samSeries = new KursDaten();
        for (int i = 0; i < this.closeDaten.size(); i++) {
            final DateValuePair orgValue = this.closeDaten.get(i);
            this.add(orgValue.getValue());
            final double gdAverage = this.getAverage();
            if (DateUtils.isInRange(orgValue.getDate(), fromDate, toDate) && !Double.isNaN(gdAverage)) {
                final DateValuePair newValue = new DateValuePair( // NOPMD
                        orgValue.getDate(), gdAverage);
                samSeries.add(newValue);
            }
        }
        result[0] = samSeries;
        return result;
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {
        return "SMA " + this.parameter.getValue(); //$NON-NLS-1$
    }

    /**
     * Aktualisiert die Summe mit angegeben Wert. Der &uml;lteste Wert aus der Queue wird abgezogen, sodass immer die
     * Summe immer nur &uuml;ber die letzten N-Werte gebildet wird.
     *
     * @param value
     */
    private void add(final double value) {
        this.sum += value;
        this.temp.add(value);
        if (this.temp.size() > this.parameter.getValue()) {
            this.sum -= this.temp.remove();
        }
    }

    /**
     * Berechnet den Durchschnitt der letzten n-Werte.
     *
     * @return den Durchschnitt der letzten n-Werte, falls in Queue schon mehr als n-1 Werte eingetragen wurden.
     */
    private double getAverage() {
        Double result = Double.NaN;
        if (this.temp.size() >= this.parameter.getValue()) {
            result = this.sum / this.temp.size();
        }
        return result;
    }
}
