/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse.Volumen; //NOPMD

import java.awt.Font;
import java.text.DecimalFormat;
import java.text.FieldPosition;
import java.time.LocalDate;
import java.util.List;

import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYBarPainter;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.willuhn.jameica.gui.Action;

/**
 * Volumen-Analysemethode.
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class Volumen extends AbstractKursAnalyse<Integer, XYPlot> {

    /**
     * Der Name der Methode
     */
    private final static String NAME = Settings.getI18n().tr("Volumen.methodName"); //$NON-NLS-1$

    /**
     * Eine Million
     */
    private final static long MILLION = 1000000;

    /**
     * Ein Tausend
     */
    private final static long TOUSEND = 1000;

    /**
     * Die TimeSerie der
     */
    private transient TimeSeries serieVol;

    /**
     * der Renderer
     */
    private transient XYBarRenderer rendererVol;

    /**
     * Liefert den Methodennamen
     *
     * @return den Methodennamen
     */
    public static String getMethodName() {
        return Volumen.NAME;
    }

    /**
     * Liefert den Action-Handler
     *
     * @return den VolumenHandleAction
     */
    public static Action getHandleAction() {
        return new VolumenHandleAction();
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {
        return Settings.getI18n().tr("Volumen.methodeTitel"); //$NON-NLS-1$
    }

    /**
     * @see AbstractKursAnalyse#getResult(LocalDate, LocalDate)
     */
    @Override
    public XYPlot getResult(final LocalDate fromDate, final LocalDate toDate) {
        if (this.anaG == null) {
            final DateAxis dateAxisKurs = new DateAxis(Settings.getI18n().tr("Volumen.axisDate")); //$NON-NLS-1$
            final NumberAxis rangeAxisKurs = new NumberAxis(Settings.getI18n().tr("Volumen.numberAxis")); //$NON-NLS-1$
            rangeAxisKurs.setAutoRangeIncludesZero(false);
            rangeAxisKurs.setNumberFormatOverride(new DecimalFormat() {

                /**
                 * Serial UID
                 */
                private static final long serialVersionUID = -7160700156383880184L;

                /**
                 * @see DecimalFormat#format(double arg0, StringBuffer arg1, FieldPosition arg2)
                 */
                @Override
                public StringBuffer format(final double number, final StringBuffer toAppendTo,
                        final FieldPosition pos) {
                    if (number >= Volumen.MILLION) {
                        toAppendTo.append((int) (number / Volumen.MILLION));
                        toAppendTo.append('m');
                    } else if (number >= Volumen.TOUSEND) {
                        toAppendTo.append((int) (number / Volumen.TOUSEND));
                        toAppendTo.append('k');
                    } else {
                        toAppendTo.append((int) number);
                    }
                    return toAppendTo;
                }
            });
            this.anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
            this.anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
            this.rendererVol = new XYBarRenderer();
            this.rendererVol.setSeriesVisibleInLegend(0, false);
            this.rendererVol.setShadowVisible(false);
            this.rendererVol.setMargin(0.85);
            this.rendererVol.setBarPainter(new StandardXYBarPainter());
            final LegendTitle legendTitle = new LegendTitle(this.anaG);
            legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
            legendTitle.setBackgroundPaint(new java.awt.Color(200, 200, 255, 100));
            legendTitle.setFrame(new BlockBorder(java.awt.Color.white));
            legendTitle.setPosition(RectangleEdge.BOTTOM);
            final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
            legendTitle.setItemLabelPadding(padding);
            final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(1.0, 0.00, legendTitle,
                    RectangleAnchor.BOTTOM_RIGHT);
            titleAnnotation.setMaxWidth(0.48);
            this.anaG.addAnnotation(titleAnnotation);
        }
        if (this.serieVol == null) {
            this.callculate();
        }
        final Day fromDay = new Day(DateUtils.asDate(fromDate));
        final Day toDay = new Day(DateUtils.asDate(toDate));
        try {
            final TimeSeriesCollection tmpMom = new TimeSeriesCollection(this.serieVol.createCopy(fromDay, toDay));
            this.anaG.setDataset(0, tmpMom);
            this.anaG.setRenderer(0, this.rendererVol);
        } catch (final CloneNotSupportedException e) {
            de.willuhn.logging.Logger.error(Settings.getI18n().tr("Volumen.logCreateTsDataError"), e); //$NON-NLS-1$
        }
        return this.anaG;
    }

    /**
     * Lass uns mal rechnen...
     */
    private void callculate() {
        this.serieVol = new TimeSeries(this.getLegendTitel());
        final List<HistoryValueBean> wphv = this.wertPapierHistory.getHistoryValues();
        // Loop thru all Values
        for (final HistoryValueBean historyValue : wphv) {
            final Day day = new Day(DateUtils.asDate(historyValue.getLocalDate())); // NOPMD
            this.serieVol.add(day, historyValue.getVolume());
        }
    }
}
